/* ASCII code <-> 2 sequential DTMF signals */
/* according to MITEL MSAN-108 Application Note */
/* MITEL MT8870, SANYO LC7385 */
/* 18-Mar-98 imaizumi@nisiq.net */

#include <stdio.h>

const unsigned char dtmf2asc_tbl[] = {
 /* SP 20 * */
 0x30, /*   0 00 */ 0x07, /* BEL 01 */ 0x51, /*   Q 02 */ 0x5A, /*   Z 03 */
 0x00, /* NUL 04 */ 0x01, /* SOH 05 */ 0x02, /* STX 06 */ 0x03, /* ETX 07 */
 0x04, /* EOT 08 */ 0x05, /* ENQ 09 */ 0x31, /*   1 10 */ 0x06, /* ACK 11 */
 0x09, /*  HT 12 */ 0x0A, /*  LF 13 */ 0x1B, /* ESC 14 */ 0x60, /*   ` 15 */
 0x27, /*   ' 16 */ 0x0B, /*  VT 17 */ 0x0C, /*  FF 18 */ 0x0D, /*  CR 19 */
 0x32, /*   2 20 */ 0x41, /*   A 21 */ 0x42, /*   B 22 */ 0x43, /*   C 23 */
 0x7F, /* DEL 24 */ 0x28, /*   ( 25 */ 0x29, /*   ) 26 */ 0x0E, /*  S0 27 */
 0x0F, /*  S1 28 */ 0x10, /* DLE 29 */ 0x33, /*   3 30 */ 0x44, /*   D 31 */
 0x45, /*   E 32 */ 0x46, /*   F 33 */ 0x08, /*  BS 34 */ 0x5C, /*   \ 35 */
 0x2F, /*   / 36 */ 0x11, /* DC1 37 */ 0x12, /* DC2 38 */ 0x13, /* DC3 39 */
 0x34, /*   4 40 */ 0x47, /*   G 41 */ 0x48, /*   H 42 */ 0x49, /*   I 43 */
 0x21, /*   ! 44 */ 0x22, /*   " 45 */ 0x2E, /*   . 46 */ 0x14, /* DC4 47 */
 0x15, /* NAK 48 */ 0x16, /* SYN 49 */ 0x35, /*   5 50 */ 0x4A, /*   J 51 */
 0x4B, /*   K 52 */ 0x4C, /*   L 53 */ 0x23, /*   # 54 */ 0x24, /*   $ 55 */
 0x25, /*   % 56 */ 0x17, /* ETB 57 */ 0x18, /* CAN 58 */ 0x19, /*  EM 59 */
 0x36, /*   6 60 */ 0x4D, /*   M 61 */ 0x4E, /*   N 62 */ 0x4F, /*   O 63 */
 0x2A, /*   * 64 */ 0x2B, /*   + 65 */ 0x2D, /*   - 66 */ 0x1A, /* SUB 67 */
 0x1C, /*  FS 68 */ 0x1D, /*  GS 69 */ 0x37, /*   7 70 */ 0x50, /*   P 71 */
 0x52, /*   R 72 */ 0x53, /*   S 73 */ 0x2C, /*   , 74 */ 0x3B, /*   ; 75 */
 0x3A, /*   : 76 */ 0x1E, /*  RS 77 */ 0x1F, /*  US 78 */ 0x26, /*   & 79 */
 0x38, /*   8 80 */ 0x54, /*   T 81 */ 0x55, /*   U 82 */ 0x56, /*   V 83 */
 0x3C, /*   < 84 */ 0x3D, /*   = 85 */ 0x3E, /*   > 86 */ 0x5B, /*   [ 87 */
 0x5D, /*   ] 88 */ 0x5F, /*   _ 89 */ 0x39, /*   9 90 */ 0x57, /*   W 91 */
 0x58, /*   X 92 */ 0x59, /*   Y 93 */ 0x3F, /*   ? 94 */ 0x40, /*   @ 95 */
 0x5E, /*   ^ 96 */ 0xFF, /* --- 97 */ 0xFF, /* --- 98 */ 0xFF  /* --- 99 */
};

/* MT8870 */
#define DTMF_1 0x1
#define DTMF_2 0x2
#define DTMF_3 0x3
#define DTMF_4 0x4
#define DTMF_5 0x5
#define DTMF_6 0x6
#define DTMF_7 0x7
#define DTMF_8 0x8
#define DTMF_9 0x9
#define DTMF_0 0xA
#define DTMF_AST 0xB
#define DTMF_SHA 0xC
#define DTMF_A 0xD
#define DTMF_B 0xE
#define DTMF_C 0xF
#define DTMF_D 0x0

/* 2 sequential DTMF signals conversion to Partial ASCII code */
/* input DTMF code High 4bit=first, low 4bit=2nd */
/* excepttion, '*' is first signal only and set to low 4bit */
char dtmf2asc(unsigned char c)
{
    int i,nible,idx;

    if (c == DTMF_AST)
        return(' ');
    for (idx=0,i=4;i>=0;i-=4) {
        nible = (c >> i)&0xf;
        if ((nible > DTMF_0) || (nible < DTMF_1))
            return('\377');
        idx *= 10;
        if (nible != DTMF_0)
            idx += nible;
    }
    return dtmf2asc_tbl[idx];
}
const unsigned char asc2dtmf_tbl[] = {
 0xA4, /*NUL 00*/ 0xA5, /*SOH 01*/ 0xA6, /*STX 02*/ 0xA7, /*ETX 03*/
 0xA8, /*EOT 04*/ 0xA9, /*ENQ 05*/ 0x11, /*ACK 06*/ 0xA1, /*BEL 07*/
 0x34, /* BS 08*/ 0x12, /* HT 09*/ 0x13, /* LF 0A*/ 0x17, /* VT 0B*/
 0x18, /* FF 0C*/ 0x19, /* CR 0D*/ 0x27, /* S0 0E*/ 0x28, /* S1 0F*/
 0x29, /*DLE 10*/ 0x37, /*DC1 11*/ 0x38, /*DC2 12*/ 0x39, /*DC3 13*/
 0x47, /*DC4 14*/ 0x48, /*NAK 15*/ 0x49, /*SYN 16*/ 0x57, /*ETB 17*/
 0x58, /*CAN 18*/ 0x59, /* EM 19*/ 0x67, /*SUB 1A*/ 0x14, /*ESC 1B*/
 0x68, /* FS 1C*/ 0x69, /* GS 1D*/ 0x77, /* RS 1E*/ 0x78, /* US 1F*/
 0x0B, /* SP 20*/ 0x44, /*  ! 21*/ 0x45, /*  " 22*/ 0x54, /*  # 23*/
 0x55, /*  $ 24*/ 0x56, /*  % 25*/ 0x79, /*  & 26*/ 0x16, /*  ' 27*/
 0x25, /*  ( 28*/ 0x26, /*  ) 29*/ 0x64, /*  * 2A*/ 0x65, /*  + 2B*/
 0x74, /*  , 2C*/ 0x66, /*  - 2D*/ 0x46, /*  . 2E*/ 0x36, /*  / 2F*/
 0xAA, /*  0 30*/ 0x1A, /*  1 31*/ 0x2A, /*  2 32*/ 0x3A, /*  3 33*/
 0x4A, /*  4 34*/ 0x5A, /*  5 35*/ 0x6A, /*  6 36*/ 0x7A, /*  7 37*/
 0x8A, /*  8 38*/ 0x9A, /*  9 39*/ 0x76, /*  : 3A*/ 0x75, /*  ; 3B*/
 0x84, /*  < 3C*/ 0x85, /*  = 3D*/ 0x86, /*  > 3E*/ 0x94, /*  ? 3F*/
 0x95, /*  @ 40*/ 0x21, /*  A 41*/ 0x22, /*  B 42*/ 0x23, /*  C 43*/
 0x31, /*  D 44*/ 0x32, /*  E 45*/ 0x33, /*  F 46*/ 0x41, /*  G 47*/
 0x42, /*  H 48*/ 0x43, /*  I 49*/ 0x51, /*  J 4A*/ 0x52, /*  K 4B*/
 0x53, /*  L 4C*/ 0x61, /*  M 4D*/ 0x62, /*  N 4E*/ 0x63, /*  O 4F*/
 0x71, /*  P 50*/ 0xA2, /*  Q 51*/ 0x72, /*  R 52*/ 0x73, /*  S 53*/
 0x81, /*  T 54*/ 0x82, /*  U 55*/ 0x83, /*  V 56*/ 0x91, /*  W 57*/
 0x92, /*  X 58*/ 0x93, /*  Y 59*/ 0xA3, /*  Z 5A*/ 0x87, /*  [ 5B*/
 0x35, /*  \ 5C*/ 0x88, /*  ] 5D*/ 0x96, /*  ^ 5E*/ 0x89, /*  _ 5F*/
 0x15, /*  ` 60*/
};

/* ASCII code conversion to 2 sequential DTMF signals */
unsigned char asc2dtmf(char c)
{
    if (c == 0x7f)
        return 0x24;
    else if ((c < 0) || (c > 0x60))
        return 0xff;
    else
        return asc2dtmf_tbl[c];
}

void main()
{
    char c;
    unsigned i;

    for (c=0;c<0x61;c++) {
      if (dtmf2asc(asc2dtmf(c))!= c)
        printf("error %02x\n",c);
    }
    c = 0x7f;
    if (dtmf2asc(asc2dtmf(c))!= c)
        printf("error %02x\n",c);

    for (i=0;i<256;i++) {
        if (asc2dtmf(dtmf2asc((unsigned char)i))!= i)
            if (((i&0xf) >= DTMF_1) && ((i&0xf) <= DTMF_0) &&
                ((i>>4) >= DTMF_1) && ((i>>4) <= DTMF_0) ) 
                printf("error %02x\n",i);
    }
}
